#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
############################################################################
#
# (c) 2006 Florian Birée aka Thesa <florian.biree@laposte.net>
# (c) 2006 Achraf Cherti aka Asher256 <achrafcherti@gmail.com>
#
#  Website: <URL:http://casetta.tuxfamily.org>
#
# Version 0.2.0
#
# changelog:
#
# # 0.2.0 version:
#
#   This file was a part of casetta in olders version. Here is the changelog
#   from the casetta CLI part:
#
#   o split casetta into the CORE part (casetta module) and the CLI part (this
#     file)
#   o rewrite the CLI part to work with the new casetta module
#   o add the --split option
#   o add the --not-interactive option
#   o add the multi-file-names input
#   o add the --help option (a syntax error just show a little message).
#   o return a non zero status when an error occur (2 for bad syntax, 1 in 
#     other case)
#   o write errors and warnings on the stderr file
#   o add a new module: prefs.py to manage the ~/.casetta settings file
#   o add the management of two new transfer tool: CaS and an user defined
#     transfert tool
#   o add the --set_default option which save the current port/transfert tool
#     as default
#   o add the --edit option to edit the settings of the user defined transfert
#     tool
#   o Disable the CaS transfer tool (CaS can be used with the other transfer
#     tool)
#   o Allow any /dev/tty* posix serial port (not only /dev/ttySn or
#      /dev/ttyUSBn)
#
# # olders version:
#
#   see casetta changelog
#
############################################################################
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#
# http://www.gnu.org/copyleft/gpl.html
#
############################################################################
"""Python module for conversions into casio programs files"""

__author__ = "Florian Birée, Achraf Cherti"
__version__ = "0.2.0"
__copyright__ = "Copyright (c) 2006, Florian Birée, Achraf Cherti"
__license__ = "GPL"
__revision__ = "$Revision: $"
# $Source: $
__date__ = "$Date: $"

# Imports
import os, sys
import casetta
import prefs

# Help
syntax = """Python module for conversions into casio programs files.

Usage: casetta [OPTION] INPUT [INPUT_FORMAT] OUTPUT [OUTPUT_FORMAT]
or:    casetta -e
       casetta -h

  OPTION :
  -s, --split           produce one file by data type in OUTPUT directory
                        with this option, you must specify OUTPUT_FORMAT.
  -n, --not_interactive casetta exit instead of asking something, and return
                        an error status
  -d, --set_default     set the current transfer tool and the current port 
                        as default.
  -e, --edit_tool       let you edit the user defined transfer tool command 
                        line. You can also edit the ~/.casetta file.
  -h, --help            show this message

  INPUT and OUTPUT must be a file name, a path, a port or the keyword 'casio',
  depend of the format. If format isn't specified, caetta will try to 
  find it thanks to the filename extension. This may fail in some case,
  just specify the format.
  The default port is the serial port 1, /dev/ttyS0 on posix, com1 on DOS.
  INPUT can be multiple file names, separated by a colon ":". In this case,
  all data from all files will be join in one file.
  INPUT_FORMAT and OUTPUT_FORMAT can be a file format, or a transfer tool.
  They can be ommited if the filename have a correct extension or if you
  want to use the default transfert tool.
  
  Available formats are:
  
  cafix     original cafix format. For output, files will be saved in the
            dirname of OUTPUT (manage programs and backups).
  cat       Cat format, used by the program Fa-12* (manage programs)
  fxi       Fxi format, used by the program Fx-Interface (only usable in 
            input, manage programs)
  newcat    New Cat format, a cat-like format with more equivalents of the 
            special characters, and unix returns to the line, to edit yours 
	        programs (manage programs)
  fxb       Fx-Interface Backup format (manage backups)
  fxm       Fx-Interface Pro Backup format (only usable in input, manage 
            backups)
  fxd       Fx-Interface Pro format (only usable in input, manage programs)
  g1r       Fa-124 format (only usable in input, manage programs)
  ctf       Calculator text format (manage programs)
  
  Reconized port:
  
  /dev/tty*     posix port * (/dev/ttySn for the linux serial port n, 
                /dev/ttyUSBn for the linux USB port n)
  comN          DOS or Miscrosoft Windows serial port N
  
  Avaible transfers tool are (after the keyword 'casio' or a port):
  
  casio     receive or send the data to the calculator using the default
            transfer tool. Optional (the keyword 'casio' is enough).
  cafix     use the transfer tool cafix. Need the command cafix in your path.
            This tool use the port /dev/casio set with the cafix-setup command.
  other     use the user defined tool. You can choose it with the -e option.
            The port of this tool depend of your configuration.

Version %s.

License: GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
""" % (__version__)

no_args_message = """
Usage: casetta [OPTION] INPUT [INPUT_FORMAT] OUTPUT [OUTPUT_FORMAT]
To know more, do: "casetta --help".
"""

def cli():
    """Run the CLI interface with arguments get by sys.argv"""
    global port, input_file, output_file
    # Check if a file name is a port path
    if input_file.startswith('/dev/tty') or input_file.startswith('com'):
        port, input_file = input_file, 'casio'
    if output_file.startswith('/dev/tty') or output_file.startswith('com'):
        port, output_file = output_file, 'casio'
    if set_default:
        prefs.define('default_port', port)
    # INPUT
    if input_file.lower() != 'casio' :
        file_data = casetta.FileData()
        for filename in input_file.split(":"):
            try :
                new_indices = file_data.add(filename, input_format)
            except casetta.FormatNotFound :
                # casetta can't find the input format
                sys.exit("casetta can't find the input format.")
            for index in new_indices :
                # check names
                if not file_data.check_name(index) and not not_interactive :
                    while not file_data.check_name(index) :
                        print "The name of the data #%s isn't correct." % \
                              str(index)
                        print 'Please enter a new name (old name:"%s").' % \
                               file_data.data[index].name
                        file_data.data[index].name = raw_input()
                elif not file_data.check_name(index) and not_interactive :
                    sys.exit("Name conflict in data# %s '%s'." % \
                             (str(index), file_data.data[index].name))
    else:
        # Import data from the calculator
        # Find the transfer tool
        if input_format == 'casio' or input_format == 'unknown' :
            tool = prefs.default_tool
        else:
            tool = 'casio_' + input_format
        if set_default:
            prefs.define('default_tool', tool)
            
        try:
            file_data = casetta.receive(tool, port)
        except casetta.ToolNotFound, tool:
            sys.exit('The transfer tool %s does not exist.' % tool)
        except casetta.NoCommand, command:
            sys.exit('The command %s is not in the PATH.' % command)
        except casetta.ToolNotConfigured:
            if tool != 'casio_cafix':
                sys.exit('An error occur using the transfer tool.')
            elif not not_interactive:
                sys.stderr.write("""
Warning: the calculator port is not configured yet.
You must run 'cafix-setup' as root.
Press CTRL-C to quit this program and do it yourself.
Press enter to run the command 'sudo cafix-setup'.
""")
                try:
                    raw_input()
                except KeyboardInterrupt:
                    sys.exit(1)
                os.system('sudo cafix-setup')
                if not os.path.exists('/dev/casio'):
                    sys.exit("/dev/casio not found.")
                else:
                    file_data = casetta.receive(tool, port)
            else:
                sys.exit("/dev/casio not found.\n" + \
                         "Run 'cafix-setup' as root to fix it.")
                         
    # OUTPUT
    if output_file != 'casio' and not split_by_data_type:
        # Save the FileData to output file
        try:
            file_data.save(output_file, output_format)
        except casetta.FormatNotFound:
            sys.exit("casetta can't find the output format.")
        except casetta.DataNoManaged, not_managed:
            if not_managed.data_type_list == ['all'] :
                sys.exit("The output format you choice can not " + \
                         "manage all input data.")
            else:
                sys.stderr.write("Warning: the output format can not manage "+\
                                 "the followed data types:\n")
                sys.stderr.write(", ".join(not_managed.data_type_list) + '\n')
                file_data.save(output_file, output_format, True)
    elif output_file != 'casio' and split_by_data_type:
        if output_format == 'unknown':
            sys.exit("casetta can't find the output format.")
        fd_dic = file_data.split_by_data_type()
        for data_key in fd_dic :
            if os.path.isdir(output_file) :
                filename = output_file
            else :
                filename = os.path.dirname(output_file)
            if type(casetta.formatExt[output_format]) != str:
                filename = os.path.join(filename, data_key + '.' + \
                                    casetta.formatExt[output_format][0])
            try:
                fd_dic[data_key].save(filename, output_format)
            except casetta.DataNoManaged, not_managed:
                if not_managed.data_type_list == ['all'] :
                    sys.stderr.write("Warning: the output format can " + \
                                     "not managed this data. Type: '%s'." % \
                                     data_key)
    else:
        # Export the FileData to the calculator
        # Find the transfer tool
        if output_format == 'casio' or output_format == 'unknown' :
            tool = prefs.default_tool
        else:
            tool = 'casio_' + output_format
        if set_default:
            prefs.define('default_tool', tool)
        
        try :
            casetta.send(file_data, tool, port)
        except casetta.ToolNotFound, tool:
            sys.exit('The transfer tool %s does not exist.' % tool)
        except casetta.NoCommand, command:
            sys.exit('The command %s is not in the PATH.' % command)
        except casetta.ToolNotConfigured:
            if tool != 'casio_cafix':
                sys.exit('An error occur using the transfer tool.')
            elif not not_interactive:
                sys.stderr.write("""
Warning: the calculator port is not configured yet.
You must run 'cafix-setup' as root.
Press CTRL-C to quit this program and do it yourself.
Press enter to run the command 'sudo cafix-setup'.
""")
                try:
                    raw_input()
                except KeyboardInterrupt:
                    sys.exit(1)
                os.system('sudo cafix-setup')
                if not os.path.exists('/dev/casio'):
                    sys.exit("/dev/casio not found.")
                else:
                    file_data = casetta.send(file_data, tool, port)
            else:
                sys.exit("/dev/casio not found.\n" + \
                         "Run 'cafix-setup' as root to fix it.")
        except casetta.DataNoManaged, not_managed:
            if not_managed.data_type_list == ['all'] :
                sys.exit("The transfert tool you choice can not " + \
                         "manage all input data.")
            else:
                sys.stderr.write("Warning: the transfer tool can not manage "+\
                                 "the followed data types:\n")
                sys.stderr.write(", ".join(not_managed.data_type_list) + '\n')
                file_data = casetta.send(file_data, tool, port, True)
    # End (print nothing)
    sys.exit(0)

def other_tool_editor():
    """Interactive editor of user defined transfer tool"""
    file_formats = []
    for format in casetta.formatType:
        if casetta.formatType[format] == 'file' and \
                casetta.openFunc.has_key(format) and \
                casetta.saveFunc.has_key(format):
            file_formats.append(format)
    print "casetta let you use your own command line transfert tool."
    print "casetta will save/read a temporary file read/save by your"
    print "transfer ttool."
    print "Enter the format of the temporary file:"
    print "(Avaible values are: " + ', '.join(file_formats) + ' )'
    format = raw_input()
    print "Enter the command line to send data to the calculator."
    print "You can use '%file' for the temporary file name, '%port'"
    print "for the serial port."
    input_cmd = raw_input()
    print "Enter the command line to receive data from the calculator."
    print "You can use '%file' for the temporary file name, '%port'"
    print "for the serial port."
    output_cmd = raw_input()
    print "Your settings are:"
    print "Format: %s" % format
    print "Input command line: %s" % input_cmd
    print "Output command line: %s" % output_cmd
    print "Write this settings? [Yn]"
    rep = raw_input()
    if rep.lower().startswith('y') or rep == '':
        prefs.define("other_tool_format", format)
        prefs.define("other_tool_input", input_cmd)
        prefs.define("other_tool_output", output_cmd)
        print "Settings saved."

if __name__ == '__main__':
    # Load settings
    prefs.load()
    if prefs.other_tool_input :
        casetta.set_other_tool(format=prefs.other_tool_format, \
                               receive_cmdline=prefs.other_tool_input, \
                               send_cmdline=prefs.other_tool_output)
    # Get arguments
    if len(sys.argv) < 2:
        sys.stderr.write(no_args_message)
        sys.exit(2)
    elif len(sys.argv) == 2 and (sys.argv[1] == '-h' or \
                                 sys.argv[1] == '--help') :
        print syntax
        sys.exit()
    elif len(sys.argv) == 2 and (sys.argv[1] == '-e' or \
                                 sys.argv[1] == '--edit') :
        other_tool_editor()
        sys.exit()
    elif len(sys.argv) < 3:
        sys.stderr.write(no_args_message)
        sys.exit(2)
    else :
        split_by_data_type = False
        not_interactive = False
        set_default = False
        input_format = 'unknown'
        output_format = 'unknown'
        port = prefs.default_port
        arg_list = []
        for arg in sys.argv[1:]:
            if arg[0] == '-' and arg[1] != '-' :
                if 's' in arg :
                    split_by_data_type = True
                if 'n' in arg :
                    not_interactive = True
                if 'd' in arg :
                    set_default = True
            elif arg == '--split':
                split_by_data_type = True
            elif arg == '--not-interactive':
                not_inter = True
            elif arg == '--set_default':
                edit = True
            else :
                arg_list.append(arg)
        if len(arg_list) == 2:
            # 0:input, 1:output
            input_file = arg_list[0]
            output_file = arg_list[1]
            cli()
        elif len(arg_list) == 3:
            # 0:input, 1:input_type|output, 2:output|output_type
            if arg_list[1].lower() in casetta.formats \
                        or ((arg_list[1].lower() == 'casio' \
                        or 'casio_' + arg_list[1].lower() in casetta.formats) \
                        and (arg_list[0].lower().startswith('com') \
                        or arg_list[0].startswith('/dev/tty'))):
                input_file = arg_list[0]
                input_format = arg_list[1].lower()
                output_file = arg_list[2]
                cli()
            else:
                input_file = arg_list[0]
                output_file = arg_list[1]
                output_format = arg_list[2].lower()
                cli()
        else:
            # 0:input, 1:input_type, 2:output, 3:output_type
            input_file = arg_list[0]
            output_file = arg_list[2]
            input_format = arg_list[1].lower()
            output_format = arg_list[3].lower()
            cli()

# vim:ai:et:sw=4:ts=4:sts=4:tw=78:fenc=utf-8
