#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
############################################################################
#
# (c) 2006 Florian Birée aka Thesa <florian.biree@laposte.net> 
# (c) 2006 Achraf Cherti aka Asher256 <achrafcherti@gmail.com>
#
# Website: <URL:http://casetta.tuxfamily.org>
# 
# Version 0.2.0
#
# changelog:
#
# # version 0.2.0:
#
# o partial rewrite of the 0.1.* version to work with casetta 0.2.*
# o New utility to find passwords in a backup
# o Export function
# o New way to show the command list: as casio menus
# o New file filter : All managed data
# o newcat is now the default save/export format
# o New preferences system (config file/preferences window)
# o The font of the program editor is now configurable
# o Syntax coloration (experimental)
# o Configuration of the syntax coloration
# o Management of base programs
# o Passwords management
# o Fix a regression in the command line file opening
# o Specials characters in data names and passwords are allowed
# o Preference window about transfers tool
# o Management of cas / user transfert tool
# o New way to choice the cafix port (allowing other port as /dev/ttyUSB0)
# o Allow the user to choice the console / sudo app.
# o Replace os.tmpnam() by os.tempnam()
# o Disable the CaS transfer tool (CaS can be used with the other transfer
#   tool)
# o new main port configuration (the same port is used by all tool). Cafix ln
#   is now made if needed when the user want to send/receive data.
# o use tempfile.mk[sd]temp instead of os.tempnam() (security reason)
# o fix bugs in the About Dialog (close button, re-open wich fails)
#
# # version 0.1.1:
#
#  o Fix a bug when the 'cancel' button is clicked in a file selection dialog
#    function on_ouvrir1_activate, on_importer1_activate and 
#    on_enregistrer_sous1_activate
#
# # version 0.1.0:
#
#   o First release
#
############################################################################
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
# 02110-1301, USA.
#
# http://www.gnu.org/copyleft/gpl.html
#
############################################################################
"""Graphic data manager for casio calculators"""

__author__ = "Florian Birée, Achraf Cherti"
__version__ = "0.2.0"
__license__ = "GPL"
__copyright__ = "Copyright (c) 2006, Florian Birée, Achraf Cherti"
__revision__ = "$Revision: $"
# $Source: $
__date__ = "$Date: $"

# i18n
application = 'casetta_gtk'
#_ = lambda text:text

# Imports
import os
import sys
import re
import gettext
gettext.install(application)
import pango
import gtk
import gtk.glade
import tempfile
import casetta
import dialogs
import fill_equiv
import prefs

# Gui classes
class MakeGui:
    """Main window class"""

    def __init__(self, filename=''):
        """Init the GUI, optionaly load filename"""
        # GUI
        self.gui = gtk.glade.XML(fname="casetta_gtk.glade", domain=application)
        
        # init variables
        self.file_changed = 0
        self.current_filename = ''
        self.current_id = -1
        
        # Creat an empty file
        self.file_data = casetta.FileData()
        
        # Load preferences and init the preferences window
        prefs.load()
        self.gui.get_widget("editor_font").set_font_name(prefs.editor_font)
        if prefs.enable_color:
            self.gui.get_widget("enable_color").set_active(True)
        else:
            self.gui.get_widget("enable_color").set_active(False)
        if prefs.color_all_commands:
            self.gui.get_widget("color_all_commands").set_active(True)
        else:
            self.gui.get_widget("color_all_commands").set_active(False)
        self.gui.get_widget("command_color").set_color(gtk.gdk.color_parse(\
                                                       prefs.command_color))
        self.gui.get_widget("variable_color").set_color(gtk.gdk.color_parse(\
                                                       prefs.variable_color))
        self.gui.get_widget("constant_color").set_color(gtk.gdk.color_parse(\
                                                       prefs.constant_color))
        self.gui.get_widget("comment_color").set_color(gtk.gdk.color_parse(\
                                                       prefs.comment_color))
        if prefs.transfer_tool and os.name == 'posix'and \
                                   command_exist('cafix'):
            prefs.define('transfer_tool', 'cafix')
        if not command_exist('cafix'):
            self.gui.get_widget("tt_cafix").set_label(_("cafix is not in \
your path"))
            self.gui.get_widget("tt_cafix").set_sensitive(False)
            self.gui.get_widget("tt_cafix_prefs").set_sensitive(False)
        if not command_exist('cas'):
            self.gui.get_widget("tt_cas").set_label(_("cas is not in \
your path"))
            self.gui.get_widget("tt_cas").set_sensitive(False)
            self.gui.get_widget("tt_cas_prefs").set_sensitive(False)
        if prefs.transfer_tool == 'cafix':
            self.gui.get_widget("tt_cafix").set_active(True)
        elif prefs.transfer_tool == 'cas':
            self.gui.get_widget("tt_cas").set_active(True)
        elif prefs.transfer_tool == 'other':
            self.gui.get_widget("tt_other").set_active(True)
        else:
            self.gui.get_widget("tt_none").set_active(True)
        self.gui.get_widget("tt_cafix_console").child.set_text( \
                                                prefs.cafix_console)
        self.gui.get_widget("tt_cafix_sudo").child.set_text(prefs.cafix_sudo)
        self.gui.get_widget("tt_port").child.set_text(prefs.port)
        self.gui.get_widget("tt_cas_console").child.set_text(prefs.cas_console)
        self.gui.get_widget("tt_other_format").child.set_text( \
                                                            prefs.other_format)
        self.gui.get_widget("tt_other_input").set_text(prefs.other_input)
        self.gui.get_widget("tt_other_output").set_text(prefs.other_output)
        # GUI Main connexions
        self.gui.signal_autoconnect(self)
        self.gui.get_widget("wmain").connect("delete_event", self.close)
        
        # Init data list
        data_list = self.gui.get_widget("dataList")
        self.data_treestore = gtk.TreeStore(int, str)
        data_list.set_model(self.data_treestore)
        # add checkbox column
        data_ccb = gtk.TreeViewColumn()
        data_ccb.set_resizable(False)
        data_list.append_column(data_ccb)
        # add the toggle renderer
        check = gtk.CellRendererToggle()
        data_ccb.pack_start(check, expand = False)
        data_ccb.add_attribute(check, "active", 0)
        check.connect("toggled", self.select_data_to_export, \
                      (self.data_treestore, 0))
        data_ccb.set_visible(True)
        # add data column
        data_tvcolumn = gtk.TreeViewColumn('Data')
        data_tvcolumn.set_resizable(False)
        data_list.append_column(data_tvcolumn)
        # add the text renderer
        data_cell = gtk.CellRendererText()
        data_tvcolumn.pack_start(data_cell, True)
        data_tvcolumn.add_attribute(data_cell, 'text', 1)
        data_tvcolumn.set_visible(True)
        self.data_items = {}
        self.data_types = {}
        self.refresh_data_list()
        
        # Init the Equiv List
        equiv_list = self.gui.get_widget("equivList")
        self.equiv_ts = gtk.TreeStore(str)
        equiv_list.set_model(self.equiv_ts)
        equiv_tvc = gtk.TreeViewColumn('equivalences')
        equiv_tvc.set_resizable(False)
        equiv_list.append_column(equiv_tvc)
        equiv_cell = gtk.CellRendererText()
        equiv_tvc.pack_start(equiv_cell, True)
        equiv_tvc.add_attribute(equiv_cell, 'text', 0)
        # Fill the Equiv list
        fill_equiv.plane(self.equiv_ts)
            
        # Init text editor
        self.editor = self.gui.get_widget("editor")
        text_buffer = self.editor.get_buffer()
        self.editor.set_wrap_mode(gtk.WRAP_NONE)
        text_buffer.connect("changed", self.on_editor_change)
        self.editor.did = -1
        self.editor.load_text = 0
        self.clipboard = gtk.Clipboard(display=gtk.gdk.display_get_default(), \
                                       selection="CLIPBOARD")
        self.editor.modify_font(pango.FontDescription(prefs.editor_font))
        # Init the syntax coloration
        tag_table = text_buffer.get_tag_table()
        variable_tag = gtk.TextTag("variable")
        variable_tag.set_property("foreground", prefs.variable_color)
        tag_table.add(variable_tag)
        command_tag = gtk.TextTag("command")
        command_tag.set_property("foreground", prefs.command_color)
        tag_table.add(command_tag)
        constant_tag = gtk.TextTag("constant")
        constant_tag.set_property("foreground", prefs.constant_color)
        tag_table.add(constant_tag)
        comment_tag = gtk.TextTag("comment")
        comment_tag.set_property("foreground", prefs.comment_color)
        tag_table.add(comment_tag)
        if prefs.color_all_commands :
            self.editor.commands = fill_equiv.make_equiv_list()
            self.editor.commands.remove('\\List ')
            self.editor.commands.remove('\\Mat ')
        else :
            self.editor.commands = fill_equiv.make_prgm_command_list()
        self.editor.command_max_len = 6
        for command in self.editor.commands:
            if len(command) > self.editor.command_max_len:
                self.editor.command_max_len = len(command)
        self.editor.variables = [chr(index) for index in range(65, 91)] + \
                                ['\\List ', '\\Mat ', '\\(r)', '\\theta']
        # Set the syntax coloration command
        self.editor_color = self.new_editor_color
        # self.editor_color = self.old_editor_color
        
        # Overwrite cafix I/O cli function in casetta
        casetta.cafix_receive_interface = cafix_receive
        casetta.cafix_send_interface = cafix_send
        casetta.cas_receive_interface = cas_receive
        casetta.cas_send_interface = cas_send
        casetta.other_receive_interface = other_receive
        casetta.other_send_interface = other_send
        
        # Select nothing
        self.select(-1)
        
        # open filename from command line
        if filename != '':
            self.open_file(filename)

    # Event manager
    def close(self, widget=None, data=None):
        """Quit the program"""
        if not self.close_file():
            return
        gtk.main_quit()
        
    def open_button(self, gui=None):
        """Show a open file window"""
        if not self.close_file():
            return
        pattern_list = []
        for format in [format for format in casetta.formats \
                       if casetta.formatType[format] == 'file' \
                       and format in casetta.openFunc]:
                       
            pattern_list.append((formats_name[format], \
                                 casetta.formatExt[format]))
        pattern_list.append((_('All files'), ['*']))
        new_filename = dialogs.open_file(pattern_list)
        if new_filename != None:
            self.open_file(new_filename)
            
    def on_data_list_click(self, treeview, event):
        """On click on the data list"""
        posx = int(event.x)
        posy = int(event.y)
        pthinfo = treeview.get_path_at_pos(posx, posy)
        if pthinfo == None:
            return
        path, col = pthinfo[0:2]
        treeview.grab_focus()
        treeview.set_cursor(path, col, 0)
        model = treeview.get_model()
        iterat = model.get_iter(path)
        content = model[iterat]
        # content[0] -> checkbox
        # content[1] -> nom
        # Find the dataID
        try :
            did = [data.name for data in self.file_data.data].index( \
                                    casetta.text_decode(content[1], True))
        except ValueError:
            did = -1
        self.current_id = did
        # Select the data
        self.select(did)
        if event.button == 3:
            #context menu
            if did == -1:
                self.gui.get_widget("sep_new").hide()
                self.gui.get_widget("find_password").hide()
                self.gui.get_widget("sep_tools").hide()
                self.gui.get_widget("renommer1").hide()
                self.gui.get_widget("supprimer2").hide()
            elif self.file_data.data[did].dType == 'unknown':
                self.gui.get_widget("sep_new").show()
                self.gui.get_widget("find_password").hide()
                self.gui.get_widget("sep_tools").hide()
                self.gui.get_widget("renommer1").hide()
                self.gui.get_widget("supprimer2").show()
            else:
                self.gui.get_widget("sep_new").show()
                self.gui.get_widget("renommer1").show()
                self.gui.get_widget("supprimer2").show()
                if self.file_data.data[did].dType == 'backup':
                    self.gui.get_widget("find_password").show()
                    self.gui.get_widget("sep_tools").show()
                else:
                    self.gui.get_widget("find_password").hide()
                    self.gui.get_widget("sep_tools").hide()
            time = event.time
            self.gui.get_widget("menu").popup(None, None, \
                                               None, event.button, time)
    
    def on_editor_change(self, gui):
        """Save the content of the editor in the file data"""
        if self.editor.did != -1 and not self.editor.load_text:
            self.file_changed = 1
            self.refresh_title()
            ebuffer = self.editor.get_buffer()
            # Synchronize with data
            self.file_data.data[self.editor.did].set_text(\
                                                ebuffer.get_text(\
                                                  ebuffer.get_start_iter(), \
                                                  ebuffer.get_end_iter()))
            # Refresh syntax coloration
            if prefs.enable_color :
                self.editor_color()

    def on_equivlist_button_press_event(self, treeview, event):
        """On click in the Equiv list"""
        posx = int(event.x)
        posy = int(event.y)
        pthinfo = treeview.get_path_at_pos(posx, posy)
        if pthinfo != None:
            path, col = pthinfo[0:2]
            treeview.grab_focus()
            treeview.set_cursor(path, col, 0)
        model = treeview.get_model()
        iterat = model.get_iter(path)
        content = model[iterat]
        # content[0] -> equiv to insert
        if not model.iter_has_child(iterat) :
            # insert equiv only if no children
            ebuffer = self.editor.get_buffer()
            ebuffer.insert_at_cursor(content[0])
        
    def about(self, gui):
        """Show the about window"""
        about_window = self.gui.get_widget("aboutdialog")
        about_window.run()
        about_window.hide()
        
    def equiv_plane_view(self, gui):
        """Show the plane view of equiv list"""
        fill_equiv.plane(self.equiv_ts)
    
    def equiv_casio_view(self, gui):
        """Show the casio view of equiv list"""
        fill_equiv.casio(self.equiv_ts)
        
    def on_base_prgm_toggled(self, gui):
        """Set if the program use base calculs"""
        if self.editor.did != -1 and not self.editor.load_text:
            self.file_data[self.editor.did].use_base = gui.get_active()
            self.file_changed = 1
            self.refresh_title()
        
    def on_password_changed(self, gui):
        """Set the program password"""
        if self.editor.did != -1 and not self.editor.load_text:
            self.file_data[self.editor.did].password = casetta.text_decode( \
                                                        gui.get_text(), True)
            self.file_changed = 1
            self.refresh_title()
        
    # GUI function
    def refresh_title(self):
        """Refresh window title"""
        window = self.gui.get_widget("wmain")
        if self.current_filename == '':
            fichier = _('No name')
        else:
            fichier = os.path.basename(self.current_filename)
        if self.file_changed:
            mod = '* '
        else:
            mod = ''
        window.set_title('%s %s- Casetta_gtk' % (fichier, mod))
        
    def refresh_data_list(self):
        """Fill/refresh the Data List"""
        self.data_treestore.clear()
        # make the dic of data type section
        data_type_i18n = {'unknown': _('Unknowns'), 'program': _('Programs'), \
                          'backup': _('Backups')}
        self.data_types = {}
        for data_type in casetta.dataType:
            if data_type != 'none':
                i18n_name = data_type
                if data_type in data_type_i18n :
                    i18n_name = data_type_i18n[data_type]
                self.data_types[data_type] = self.data_treestore.\
                                              append(None,[False, i18n_name])
        #Fill childrens items
        self.data_items = {}
        for index in range(len(self.file_data.data)) :
            if index in self.file_data.export:
                export = True
            else: 
                export = False
            self.data_items[index] = self.data_treestore.\
                        append(self.data_types[self.file_data[index].dType], \
                               [export, casetta.text_encode( \
                               self.file_data[index].name, True)])

    # File management function
    def new_file(self, gui):
        """Make a new file"""
        if not self.close_file():
            return
        self.file_data = casetta.FileData()
        self.file_changed = 0
        self.current_filename = ''
        self.refresh_title()
        self.refresh_data_list()
        
    def open_file(self, filename):
        """Open a file"""
        try:
            file_data = casetta.open_file(filename)
        except casetta.FormatCantOpen:
            dialogs.error(_("The format of this file wasn't detected. \
\nIt wont be opened."))
            return None
        if file_data == None:
            dialogs.error(_("The file cannot be opened."))
            return None
        else :
            self.file_data = file_data
            self.current_filename = filename
            self.refresh_title()
            self.refresh_data_list()
            
    def close_file(self):
        """Close the current file"""
        self.select(-1)
        if self.file_changed:
            if dialogs.question(_('Do you want to save this file?')):
                #yes
                self.save_file()
            else:
                #no
                return 1
        else:
            return 1
        #return 0 : don't close
        
    def save_file(self, gui=None):
        """Save the current file"""
        if self.current_filename == '':
            #Save as
            self.save_file_as()
        else:
            self.file_data.save(self.current_filename)
            self.file_changed = 0
            self.refresh_title()
                    
    def save_file_as(self, gui=None):
        """Save as the current file"""
        pattern_list = []
        pattern_list.append((formats_name['newcat'], \
                                 casetta.formatExt['newcat']))
        for format in [format for format in casetta.formats \
                       if casetta.formatType[format] == 'file' \
                       and format in casetta.saveFunc
                       and format != 'newcat']:
            pattern_list.append((formats_name[format], \
                                 casetta.formatExt[format]))
        new_filename = dialogs.save_file(pattern_list)
        if new_filename != None:
            self.current_filename = new_filename
            try:
                self.file_data.save(self.current_filename)
            except casetta.FormatNotFound:
                dialogs.error(_("Casetta_gtk can't find the file format. \
Please specify it."))
                return
            except casetta.DataNoManaged, not_managed:
                if not_managed.data_type_list == ['all'] :
                    dialogs.error(_("The output format you choice can not \
 manage all input data.\nPlease choose another output format."))
                    return
                else:
                    dialogs.question(_("Warning: the output format can not \
manage the followed data types: %s. Do you really want to save the \
current file in this format?") % ", ".join(not_managed.data_type_list))
                self.file_data.save(filename=self.current_filename, \
                                    ignore_warnings=True)
            self.file_changed = 0
            self.refresh_title()
    
    def import_file(self, gui):
        """Import a file"""
        pattern_list = []
        for format in [format for format in casetta.formats \
                       if casetta.formatType[format] == 'file' \
                       and format in casetta.openFunc]:
                       
            pattern_list.append((formats_name[format], \
                                 casetta.formatExt[format]))
        pattern_list.append((_('All files'), ['*']))
        new_filename = dialogs.open_file(pattern_list)
        if new_filename != None:
            try:
                new_indices = self.file_data.add(new_filename)
            except casetta.FormatCantOpen:
                dialogs.error(_("The format of this file wasn't detected. \
\nIt wont be opened."))
                return
            diff = 0
            for index in new_indices :
                new_index = self.check_name(index + diff)
                if new_index == -1:
                    diff -= 1
            self.refresh_data_list()
            self.file_changed = 1
            self.refresh_title()
            
    def export(self, gui=None):
        """Export selected data to a file"""
        # Check if some data are selected
        if self.file_data.export == []:
            dialogs.error(_("You haven't choice data to export. \
Select first data by using the checkboxes in the list to \
the left. Next retry to export."))
            return
        # Select a file to export
        pattern_list = []
        pattern_list.append((formats_name['newcat'], \
                                 casetta.formatExt['newcat']))
        for format in [format for format in casetta.formats \
                       if casetta.formatType[format] == 'file' \
                       and format in casetta.saveFunc
                       and format != 'newcat']:
            pattern_list.append((formats_name[format], \
                                 casetta.formatExt[format]))
        new_filename = dialogs.save_file(pattern_list)
        # Export
        if new_filename != None:
            self.current_filename = new_filename
            try:
                self.file_data.save(self.current_filename, export=True)
            except casetta.FormatNotFound:
                dialogs.error(_("Casetta_gtk can't find the file format. \
Please specify it."))
                return
            except casetta.DataNoManaged, not_managed:
                if not_managed.data_type_list == ['all'] :
                    dialogs.error(_("The output format you choice can not \
 manage all input data.\nPlease choose another output format."))
                    return
                else:
                    dialogs.question(_("Warning: the output format can not \
manage the followed data types: %s. Do you really want to save the \
current file in this format?") % ", ".join(not_managed.data_type_list))
                self.file_data.save(filename=self.current_filename, \
                                    ignore_warnings=True, export=True)

    # Data management functions
    def check_name(self, data_index, rename=False):
        """Check the name of a data
        
        You can specify the rename flag to True to rename the data
        If the name not matches the rules, re-ask
        If the name is already used, ask to erase the old
        Return the id of the renamed data
        
        """
        old_name = self.file_data[data_index].name
        name_is_good = 0
        index_to_del = -1
        if rename:
            force_input = 1
        else:
            force_input = 0
        while not name_is_good:
            # check if already employed
            if force_input:
                name_is_good = 0
                force_input = 0
            else :
                name_is_good = 1
            for index in range(len(self.file_data.data)) :
                if index != data_index and \
                        self.file_data.data[index].dType == self.file_data.\
                        data[data_index].dType and \
                        self.file_data.data[index].name == self.file_data.\
                        data[data_index].name :
                    if dialogs.question(_("This name is already used. \
Do you want to replace the other data?")):
                        # yes -> replace -- delete index, data_id - 1
                        index_to_del = index
                        data_index -= 1
                        name_is_good = 1
                    else:
                        # no -> ask a new name
                        name_is_good = 0
            # check the syntax of the name
            if re.search(casetta.dataName.get(self.file_data[data_index].\
                                            dType), self.file_data\
                                            [data_index].name) == None :
                name_is_good = 0
            # if name is not good, ask
            if not name_is_good:
                # identify the data form
                data_form = ''
                regex = casetta.dataName[self.file_data[data_index].dType]
                limited_string = re.compile(r'^\^\.\{1,(.+)\}\$$')
                if limited_string.search(regex) != None :
                    (nbr_letters,) = limited_string.search(regex).groups()
                    data_form = _('(maximum %s letters)') % nbr_letters
                new_name = casetta.text_decode(dialogs.input_box( \
                               _('New name'), _("Please specify \
a new name for this data. %s") %\
                               data_form, casetta.text_encode( \
                               self.file_data[data_index].name, True)), True)
                if new_name == '' and not rename :
                    self.file_data.del_data(data_index)
                    return -1
                elif new_name == '' and rename :
                    self.file_data[data_index].name = old_name
                    return data_index
                self.file_data[data_index].name = new_name
        if index_to_del != -1:
            self.file_data.del_data(index_to_del)
        return data_index
       
        
    def new_program(self, gui=None):
        """Make a new program"""
        did = self.file_data.new_data()
        self.file_data.data[did].dType = 'program'
        did = self.check_name(did)
        self.refresh_data_list()
        self.file_changed = 1
        self.refresh_title()
        
    def rename_data(self, gui=None):
        """Rename a data"""
        if self.current_id == -1:
            return
        self.check_name(self.current_id, True)
        self.refresh_data_list()
        self.file_changed = 1
        self.refresh_title()
        self.select(-1)
        
    def delete_data(self, gui):
        """Delete a data"""
        if self.current_id == -1:
            return
        if dialogs.question(_('Do you really want to delete the data "%s"?') %\
              casetta.text_encode(self.file_data[self.current_id].name, True)):
            self.file_data.del_data(self.current_id)
            self.refresh_data_list()
            self.file_changed = 1
            self.refresh_title()
            self.select(-1)
            
    def select_data_to_export(self, gui, path, user_data):
        """Add/Remove a data to/from the export list"""
        if self.current_id == -1:
            return
        model, column = user_data
        model[path][column] = not model[path][column]
        if model[path][column]:
            #Check if self.current_id is in self.file_data.export
            if not self.current_id in self.file_data.export:
                self.file_data.export.append(self.current_id)
        else:
            #Check if self.current_id is NOT in self.file_data.export
            if self.current_id in self.file_data.export:
                self.file_data.export.remove(self.current_id)

    # Data tools, viewer and editors functions
    def select(self, did):
        """Change the window follow the selected data"""
        text_frame = self.gui.get_widget("vbox6")
        label_frame = self.gui.get_widget("info")
        equiv_frame = self.gui.get_widget("vbox3")
        #If nothing is selected (did=-1)
        if did == -1:
            text_frame.hide()
            label_frame.show()
            equiv_frame.hide()
            label_frame.set_text(_("Select data in the left list to edit \
it or obtain informations on it.\nYou can use the contextual menu (right \
click) to creat new data."))
            self.editor.did = -1
        else:
            # Extract date
            if self.file_data.data[did].date != None:
                date = self.file_data[did].date.strftime('%d %b %Y')
            else:
                date = _('N/A')
            if self.file_data.data[did].dType == 'program':
                text_frame.show()
                label_frame.hide()
                equiv_frame.show()
                self.edit_text_data(did)
            elif self.file_data.data[did].dType == 'backup':
                text_frame.hide()
                label_frame.show()
                equiv_frame.hide()
                label_frame.set_text(\
                           _("Backup\nName: %(name)s\nDate: %(date)s\n") %\
                           {'name': self.file_data.data[did].name, \
                           'date': date})
            else:
                # others data
                text_frame.hide()
                label_frame.show()
                equiv_frame.hide()
                label_frame.set_text(\
                       _("Unknown data type\nName: %(name)s\nDate: %(date)s\n \
Size: %(size)d") % {'name': self.file_data.data[did].name, \
                    'date': date, \
                    'size': len(self.file_data.data[did].raw_data)})
                self.editor.did = -1
                
    def edit_text_data(self, did):
        """Open a program in the editor"""
        ebuffer = self.editor.get_buffer()
        self.editor.did = did
        self.editor.load_text = 1
        ebuffer.set_text(self.file_data.data[did].get_text())
        self.gui.get_widget('base_prgm').set_active( \
                                     self.file_data[did].use_base)
        self.gui.get_widget('prgm_password').set_text( \
                    casetta.text_encode(self.file_data[did].password, True))
        self.editor.load_text = 0
        # Syntax coloration for all the text
        if prefs.enable_color :
            self.editor_color()
        
    def cut_text(self, gui):
        """Cut in the program editor (menu entry)"""
        if self.editor.did == -1:
            return
        ebuffer = self.editor.get_buffer()
        ebuffer.cut_clipboard(self.clipboard, self.editor.get_editable())
        
    def copy_text(self, gui):
        """Copy in the program editor (menu entry)"""
        if self.editor.did == -1:
            return
        ebuffer = self.editor.get_buffer()
        ebuffer.copy_clipboard(self.clipboard)
        
    def paste_text(self, gui):
        """Paste in the program editor (menu entry)"""
        if self.editor.did == -1:
            return
        ebuffer = self.editor.get_buffer()
        ebuffer.paste_clipboard(self.clipboard, None, \
                                self.editor.get_editable())
        
    def delete_text(self, gui):
        """Delete in the program editor (menu entry)"""
        if self.editor.did == -1:
            return
        ebuffer = self.editor.get_buffer()
        ebuffer.delete_selection(True, self.editor.get_editable())
        
    def new_editor_color(self, line=None):
        """Color the text following the syntax (New way)"""
        text_buffer = self.editor.get_buffer()
        if line != None:
            start = text_buffer.get_iter_at_line(line)
            limit = start.copy()
            limit.forward_to_line_end()
        else:
            start = text_buffer.get_start_iter()
            limit = text_buffer.get_end_iter()
        
        while not start.compare(limit) > -1:
            second = start.copy()
            step = 0
            while step < self.editor.command_max_len:
                second.forward_char()
                step += 1
                if start.get_text(second) == "'":
                    # comment
                    second.forward_to_line_end()
                    text_buffer.apply_tag_by_name("comment", start, second)
                    start = second
                    break
                elif start.get_text(second) == '"':
                    # string
                    try:
                        second = second.forward_search('"', \
                                          gtk.TEXT_SEARCH_TEXT_ONLY, limit)[1]
                    except TypeError:
                        break
                    text_buffer.apply_tag_by_name("constant", start, second)
                    start = second
                    break
                elif start.get_text(second).isdigit():
                    # digit
                    text_buffer.apply_tag_by_name("constant", start, second)
                    start = second
                    break
                elif start.get_text(second) in self.editor.variables:
                    # variables
                    text_buffer.apply_tag_by_name("variable", start, second)
                    start = second
                    break
                elif start.get_text(second) in self.editor.commands:
                    # commands
                    text_buffer.apply_tag_by_name("command", start, second)
                    start = second
                    break
            if not start == second:
                start.forward_char()
        
    def old_editor_color(self, line=None):
        """Color the text following the syntax (old way)"""
        text_buffer = self.editor.get_buffer()
        if line != None:
            start = text_buffer.get_iter_at_line(line)
            limit = start.copy()
            limit.forward_to_line_end()
        else:
            start = text_buffer.get_start_iter()
            limit = text_buffer.get_end_iter()

        # search commands
        for command in self.editor.commands :
            match_end = start.copy()
            while 1:
                try:
                    match_start, match_end = match_end.forward_search( \
                                   command, gtk.TEXT_SEARCH_TEXT_ONLY, limit)
                except TypeError:
                    break
                text_buffer.apply_tag_by_name("command", match_start, \
                                                            match_end)
        # search variables
        for variable in self.editor.variables :
            match_end = start.copy()
            while 1:
                try:
                    match_start, match_end = match_end.forward_search( \
                                  variable, gtk.TEXT_SEARCH_TEXT_ONLY, limit)
                except TypeError:
                    break
                text_buffer.apply_tag_by_name("variable", match_start, \
                                              match_end)
        # search constants (0~1, "strings")
        for digit in range(10) :
            match_end = start.copy()
            while 1:
                try:
                    match_start, match_end = match_end.forward_search( \
                                str(digit), gtk.TEXT_SEARCH_TEXT_ONLY, limit)
                except TypeError:
                    break
                text_buffer.apply_tag_by_name("constant", match_start, \
                                              match_end)
        match_end = start.copy()
        while 1:
            try:
                match_start, match_end = match_end.forward_search( \
                                       '"', gtk.TEXT_SEARCH_TEXT_ONLY, limit)
            except TypeError:
                break
            try:
                match_end = match_end.forward_search('"', \
                                          gtk.TEXT_SEARCH_TEXT_ONLY, limit)[1]
            except TypeError:
                break
            text_buffer.apply_tag_by_name("constant", match_start, match_end)
        # search comments (' to the end of line)
        match_end = start.copy()
        while 1:
            try:
                match_start, match_end = match_end.forward_search("'", \
                                           gtk.TEXT_SEARCH_TEXT_ONLY, limit)
            except TypeError:
                break
            match_end.forward_to_line_end()
            text_buffer.apply_tag_by_name("comment", match_start, match_end)
        
    def find_password(self, gui=None):
        """Find password for a program in a backup"""
        if self.current_id == -1:
            return
        if self.file_data[self.current_id].dType != 'backup' :
            return
        prgm_name = dialogs.input_box(title=_("Find password"), 
                                      message="Please enter the \
name of the program you want to find the password. (Program name is case \
sensitive.)", default_text='', modal= True)
        if prgm_name == '':
            return
        
        password_pattern = re.compile('''
        [\x00\xee]      # first character (0x00 except if the last prgm have a     
                        # password. In this case, 0xee)
        %s              # the program name
        \xff{0,7}       # to complete the program name to 8 character
        \x00{5}         # 5 \x00 character
        .{2}            # 2 character
        \x10            # one 0x10 character
        ([^\xff]{1,8})  # 1 to 8 non 0xff character : the password
        \xff{0,7}       # to complete the password to 8 character
        \x00{7}         # 6 \x00 characters
        \xee            # One 0xee character
        ''' % prgm_name, re.VERBOSE)
        password_list = password_pattern.findall(self.file_data\
                                                 [self.current_id].raw_data)
        if password_list == [] :
            dialogs.info(_('No password have been found for the program \
"%s".') % prgm_name)
        else:
            dialogs.info(_('Passwords found for the programe "%(prgm_name)s": \
"%(passwd_list)s".')% {'prgm_name': prgm_name, \
                       'passwd_list': '", "'.join(password_list)})
    
    # Generic device management function            
    def receive(self, gui):
        """import data from a casio calculator"""
        if prefs.transfer_tool == 'none' or prefs.transfer_tool == '' :
            dialogs.error(_("You must select a transfert tool first in the \
preference window."))
            return
        if prefs.transfer_tool == 'cafix' and not cafix_check_port():
            return
        port = prefs.port
        if prefs.transfer_tool == 'other':
            casetta.set_other_tool(prefs.other_format, prefs.other_input, \
                                   prefs.other_output)
        # Import data
        try:
            in_data = casetta.receive('casio_' + prefs.transfer_tool, port)
        except casetta.NoCommand, command:
            dialogs.error(_("The command '%s' is not in your path. To use \
the current transfer tool, you must install this command.") % command)
            return
        except casetta.ToolNotConfigured:
            dialogs.error(_("An error occur during the transfer. Check the \
configuration of your transfer tool."))
            return
        new_indices = self.file_data.add_data(in_data)
        diff = 0
        for index in new_indices :
            new_index = self.check_name(index + diff)
            if new_index == -1:
                diff -= 1
            self.refresh_data_list()
            self.file_changed = 1
            self.refresh_title()

    def send(self, gui):
        """send data to a casio calculator"""
        if prefs.transfer_tool == 'none' or prefs.transfer_tool == '' :
            dialogs.error(_("You must select a transfert tool first in the \
preference window."))
            return
        if prefs.transfer_tool == 'cafix' and not cafix_check_port():
            return
        port = prefs.port
        if prefs.transfer_tool == 'other':
            casetta.set_other_tool(prefs.other_format, prefs.other_input, \
                                   prefs.other_output)
         # Check if some data are selected
        if self.file_data.export == []:
            dialogs.error(_("You haven't choice data to send to your \
calculator. Select first data by using the checkboxes in the list to \
the left. Next retry to send data."))
            return
        #Print the 'select receive state on casio' message
        if not dialogs.info(_("Run your calculator, and set it in \
the state 'Receiving'."), True):
            return
        # Export data
        try:
            casetta.send(self.file_data, 'casio_' + prefs.transfer_tool, \
                         port, True, True)
        except casetta.NoCommand, command:
            dialogs.error(_("The command '%s' is not in your path. To use \
the current transfer tool, you must install this command.") % command)
            return
        except casetta.ToolNotConfigured:
            dialogs.error(_("An error occur during the transfer. Check the \
configuration of your transfer tool."))
            return
            
    # Preference window function
    def wprefs_open(self, gui):
        """Open the preference window"""
        self.gui.get_widget("wprefs").show()
        
    def wprefs_close(self, gui):
        """Close the preference window"""
        self.gui.get_widget("wprefs").hide()

    def wprefs_font_set(self, gui):
        """Change the font of the program editor"""
        font = gui.get_font_name()
        prefs.define('editor_font', font)
        self.editor.modify_font(pango.FontDescription(font))
        
    def wprefs_enable_color(self, gui):
        """Enable/disable the syntax coloration"""
        if gui.get_active():
            prefs.define('enable_color', '1')
            self.editor_color()
        else:
            prefs.define('enable_color', '0')

    def wprefs_color_all_commands(self, gui):
        """Enable/disable the coloration of all commands"""
        if gui.get_active():
            prefs.define('color_all_commands', '1')
            self.editor.commands = fill_equiv.make_equiv_list()
            self.editor.commands.remove('\\List ')
            self.editor.commands.remove('\\Mat ')
        else:
            prefs.define('color_all_commands', '0')
            self.editor.commands = fill_equiv.make_prgm_command_list()
        self.editor.command_max_len = 6
        for command in self.editor.commands:
            if len(command) > self.editor.command_max_len:
                self.editor.command_max_len = len(command)
        self.editor_color()
            
    def wprefs_command_color_set(self, gui):
        """Change the color of commands"""
        color = gdk_color_to_hexa(gui.get_color())
        prefs.define('command_color', color)
        tag = self.editor.get_buffer().tag_table.lookup('command')
        tag.set_property("foreground", color)
        self.editor_color()
        
    def wprefs_variable_color_set(self, gui):
        """Change the color of variable"""
        color = gdk_color_to_hexa(gui.get_color())
        prefs.define('variable_color', color)
        tag = self.editor.get_buffer().tag_table.lookup('variable')
        tag.set_property("foreground", color)
        self.editor_color()
        
    def wprefs_constant_color_set(self, gui):
        """Change the color of constant"""
        color = gdk_color_to_hexa(gui.get_color())
        prefs.define('constant_color', color)
        tag = self.editor.get_buffer().tag_table.lookup('constant')
        tag.set_property("foreground", color)
        self.editor_color()
        
    def wprefs_comment_color_set(self, gui):
        """Change the color of comment"""
        color = gdk_color_to_hexa(gui.get_color())
        prefs.define('comment_color', color)
        tag = self.editor.get_buffer().tag_table.lookup('comment')
        tag.set_property("foreground", color)
        self.editor_color()
        
    def wprefs_tool_changed(self, gui):
        """Change the current transfer tool"""
        if gui.state == 2:
            prefs.define('transfer_tool', gui.get_name()[3:])
    
    def wprefs_tool_settings_changed(self, gui):
        """Change a setting about transfers tool"""
        if 'child' in dir(gui):
            opt_name = gui.get_name()[3:]
            value = gui.child.get_text()
        else :
            opt_name = gui.get_name()[3:]
            value = gui.get_text()
        prefs.define(opt_name, value)

# cafix management function    
def cafix_check_port() :
    """Check if /dev/casio exist.
    
    If not, ask to make the link as root.
    Return True if /dev/casio is Ok, False else.
    
    """
    if not os.path.exists('/dev/casio'):
        rep = dialogs.question(_("Your calculator port is not set in the cafix \
configuration. Do you want to set the serial port from the preferences window \
in the cafix configuration (this must be as root and at each reboot) ?"))
        if rep :
            command = prefs.cafix_sudo + ' "ln -s -f ' + prefs.port + ' /dev/casio"'
            os.system(command)
            return os.path.exists('/dev/casio')
        else:
            return False
    else:
        return True

def cafix_receive(tmp_folder):
    """Receive data from a casio using cafix"""
    script = open(os.path.join(tmp_folder,'receive.sh'),'w')
    ctn = "#!/bin/sh\n"
    ctn += "# Receive script made by casetta_gtk\n"
    ctn += "cd %s\n" % tmp_folder
    ctn += "echo %s\n" % _('You can now send data from the calculator')
    ctn += "cafix -r\n"
    script.write(ctn)
    script.close()
    # Run tmp_folder/receive.sh in a console
    os.system(prefs.cafix_console + " 'sh %s'" % \
              os.path.join(tmp_folder,'receive.sh'))
    
def cafix_send(tmp_folder):
    """Send data to a casio using cafix"""
    script_filename = tempfile.mkstemp('.sh')[1]
    script = open(script_filename,'w')
    ctn = "#!/bin/sh\n"
    ctn += "# Send script made by casetta_gtk\n"
    ctn += "cd %s\n" % tmp_folder
    ctn += "cafix -s *\n"
    script.write(ctn)
    script.close()
    # Run script in a console
    os.system(prefs.cafix_console + " 'sh %s'" % script_filename)

# cas management function
def cas_receive(ctf_file, port):
    """Receive data from a casio using CaS"""
    os.system(prefs.cas_console + " 'cas -i " + port + " -o \"" + ctf_file + \
              "\"'")
    
def cas_send(ctf_file, port):
    """Send data to a casio using CaS"""
    os.system(prefs.cas_console + " 'cas -i \"" + ctf_file + "\" -o " + \
              port + "'")

# other transfer tool management function
def other_receive(cmdline):
    """Receive data from a casio using another transfer tool"""
    os.system(cmdline)
    
def other_send(cmdline):
    """Send data to a casio using another transfer tool"""
    os.system(cmdline)

# Generic functions
def command_exist(command) :
    """Check if command is in the path"""
    if len([path for path in os.defpath.split(os.pathsep) \
                 if os.path.exists(os.path.join(path,command))]) == 0:
        return False
    else:
        return True

def gdk_color_to_hexa(gdk_color):
    """Convert a gtk.gdk.Color into a hexa string"""
    red = hex(gdk_color.red)[2:]
    if len(red) == 1:
        red = '0' + red
    green = hex(gdk_color.green)[2:]
    if len(green) == 1:
        green = '0' + green
    blue = hex(gdk_color.blue)[2:]
    if len(blue) == 1:
        blue = '0' + blue
    hexa = '#' + red + green + blue
    return hexa

formats_name = {'newcat': _('Newcat format'), \
                'cafix': _('Cafix files'), \
                'cat': _('Fa-122/Fa-123 files'), \
                'fxi': _('Fx-Interface files'), \
                'fxb': _('Fx-Interface backups'), \
                'fxm': _('Fx-Interface Pro backups'), \
                'fxd': _('Fx-Interface Pro files'), \
                'g1r': _('Fa-124 files'), \
                'ctf': _('Calculator Text Format')}
for format_id in [format_id for format_id in casetta.formats \
                       if casetta.formatType[format_id] == 'file']:
    if not formats_name.has_key(format_id):
        formats_name[format_id] = _('%s files') % format_id

# Main code
if __name__ == "__main__":
    if len(sys.argv) == 1:
        cli_filename = ''
    elif sys.argv[1].lower() == '-h' or sys.argv[1].lower() == '--help':
        # CLI help text
        print 'Casetta_gtk -- Graphic interface for casetta'
        print 'Usage: casetta_gtk [filename]'
        print 'Open filename in Casetta_gtk'
        print "You can also use 'casetta_gtk --help' to display this message"
        sys.exit()
    else:
        # check if sys.argv[1] is really a file
        if os.path.isfile(sys.argv[1]):
            cli_filename = sys.argv[1]
        else:
            sys.stderr.write('Bad filename. Run casetta_gtk normally.\n')
            cli_filename = ''
    ui = MakeGui(cli_filename)
    gtk.main()

# vim:ai:et:sw=4:ts=4:sts=4:tw=78:fenc=utf-8
