# -*- coding: utf-8 -*-
#
############################################################################
#
# (c) 2006 Florian Birée aka Thesa <florian.biree@laposte.net> 
# (c) 2006 Achraf Cherti aka Asher256 <achrafcherti@gmail.com>
#
# Website: <URL:http://casetta.tuxfamily.org>
# 
# Version 0.2.0
#
# changelog:
#
# # version 0.2.0:
#
# o first version (version number aligned on casetta_gtk)
#
############################################################################
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
# 02110-1301, USA.
#
# http://www.gnu.org/copyleft/gpl.html
#
############################################################################
"""Preference input/output functions for casetta_gtk"""

__author__ = "Florian Birée, Achraf Cherti"
__version__ = "0.2.0"
__license__ = "GPL"
__copyright__ = "Copyright (c) 2006, Florian Birée, Achraf Cherti"
__revision__ = "$Revision: $"
# $Source: $
__date__ = "$Date: $"

# Imports
import os
import ConfigParser

# Functions
def load():
    """Load a config file"""
    global editor_font, enable_color, color_all_commands, command_color, \
           variable_color, constant_color, comment_color, transfer_tool, \
           cafix_console, cafix_sudo, cas_console, port, other_format, \
           other_input, other_output
    try:
        config = ConfigParser.ConfigParser()
        config.read(config_file)
        editor_font = config.get("Program editor", "font")
        enable_color = int(config.get("Program editor", "enable color"))
        color_all_commands = int(config.get("Program editor", \
                                            "color all command"))
        command_color = config.get("Program editor", "command color")
        variable_color = config.get("Program editor", "variable color")
        constant_color = config.get("Program editor", "constant color")
        comment_color = config.get("Program editor", "comment color")
        
        transfer_tool = config.get("Transfer tool", "transfer tool")
        port = config.get("Transfer tool", "port")
        
        cafix_console = config.get("Transfer tool: cafix", "console")
        cafix_sudo = config.get("Transfer tool: cafix", "sudo")
        
        cas_console = config.get("Transfer tool: cas", "console")
        
        other_format = config.get("Transfer tool: other", "format")
        other_input = config.get("Transfer tool: other", "receive command")
        other_output = config.get("Transfer tool: other", "send command")
    except IOError:
        return
    except ConfigParser.NoOptionError:
        return
    except ConfigParser.NoSectionError:
        return

def define(name, val):
    """Change the setting name to the value val"""
    global editor_font, enable_color, color_all_commands, command_color, \
           variable_color, constant_color, comment_color, transfer_tool, \
           cafix_console, cafix_sudo, cas_console, port, other_format, \
           other_input, other_output
    globals()[name] = val
    afile = open(config_file, 'w' )
    config = ConfigParser.ConfigParser()
    config.add_section("Program editor")
    config.set("Program editor", "font", editor_font)
    config.set("Program editor", "enable color", str(enable_color))
    config.set("Program editor", "color all command", str(color_all_commands))
    config.set("Program editor", "command color", command_color)
    config.set("Program editor", "variable color", variable_color)
    config.set("Program editor", "constant color", constant_color)
    config.set("Program editor", "comment color", comment_color)
    config.add_section("Transfer tool")
    config.set("Transfer tool", "transfer tool", transfer_tool)
    config.set("Transfer tool", "port", port)
    config.add_section("Transfer tool: cafix")
    config.set("Transfer tool: cafix", "console", cafix_console)
    config.set("Transfer tool: cafix", "sudo", cafix_sudo)
    config.add_section("Transfer tool: cas")
    config.set("Transfer tool: cas", "console", cas_console)
    config.add_section("Transfer tool: other")
    config.set("Transfer tool: other", "format", other_format)
    config.set("Transfer tool: other", "receive command", other_input)
    config.set("Transfer tool: other", "send command", other_output)
    config.write(afile)
    afile.close()

# Default values
config_file = os.path.join(os.path.expanduser('~'), '.casetta_gtk')
# [Program editor] section
editor_font = "Courier 10 Pitch 12"
enable_color = 0
color_all_commands = 0
command_color = "#A52A2A"
variable_color = "#2E8B57"
constant_color = "#FF00FF"
comment_color = "#0000FF"
# [Transfer tool] section
transfer_tool = ''
port = '/dev/ttyS0'
# [Transfer tool: cafix] section
cafix_console = 'gnome-terminal -e'
cafix_sudo = 'gksu'
# [Transfer tool: cas] section
if os.name == 'posix':
    cas_console = 'gnome-terminal -e'
else:
    cas_console = 'command /c'
# [Transfer tool: other] section
other_format = ''
other_input = ''
other_output = ''
